/* ps.cc

   Copyright 1996, 1997, 1998 Cygnus Solutions.

This file is part of Cygwin32.

This software is a copyrighted work licensed under the terms of the
Cygwin32 license.  Please consult the file "CYGWIN32_LICENSE" for
details. */

#include <stdio.h>
#include <time.h>
#include <string.h>
#include "winsup.h"
#include "external.h"

static char *
start_time (external_pinfo *child)
{
  time_t st = child->start_time;
  time_t t = time (NULL);
  static char stime[40] = {'\0'};
  char now[40];

  strncpy (stime, ctime (&st) + 4, 15);
  strcpy (now, ctime (&t) + 4);
  if ((t - st) < (24 * 3600))
      return stime + 7;
  stime[6] = '\0';
  return stime;
}

int
main (int argc, char *argv[])
{
  BOOL full;
  char *title =   "  %5s %5s %5s %6s %4s %3s %8s %s\n";
  char *format = "%c %5d %5d %5d %6d %4d %3d %8s %s\n";
  external_pinfo *p;

  printf (title, "PID", "PPID", "PGID", "WINPID", "UID", "TTY",
	  "STIME", "COMMAND");

  if (argv[1] && strcmp (argv[1], "-f") == 0)
    full = TRUE;
  else
    full = FALSE;

  (void) cygwin32_internal (CW_LOCK_PINFO, 1000);
  for (int pid = 0;
       p = (external_pinfo *)cygwin32_internal (CW_GETPINFO, pid|CW_NEXTPID);
       pid = p->pid)
    {
      if (p->process_state == PID_NOT_IN_USE ||
	  (!full && kill (p->pid, 0)))
	continue;
      char status = ' ';
      if (p->process_state & PID_STOPPED)
	status = 'S';
      else if (p->process_state & PID_TTYIN)
	status = 'I';
      else if (p->process_state & PID_TTYOU)
	status = 'O';
      printf (format, status, p->pid, p->ppid, p->pgid,
	      p->dwProcessId, p->uid, p->ctty, start_time (p),
	      (p->process_state & PID_ZOMBIE) ? "<defunct>" : p->progname);
    }
  (void) cygwin32_internal (CW_UNLOCK_PINFO);

  return 0;
}
